
-- locomotive joint examples

viewer.mode=STANDARD

function setup()
    rectMode(CENTER)
    parameter.integer("pressure",0,200,20)
    parameter.boolean("break-piston-joint",false)
    parameter.boolean("break_joint1_2",false)
    parameter.boolean("break_joint2_3",false)
    parameter.boolean("break_joint3_4",false)
    
    -- create wheels
    w1=wheel(200,200,200,200,200,260)
    w2=wheel(380,200,380,200,380,260)
    w3=wheel(560,200,560,200,560,260)
    w4=wheel(740,200,740,200,740,260)
    
    createPiston()
    createJoints()
end

function draw()
    background(0)
    
    if CurrentOrientation~=LANDSCAPE_LEFT and CurrentOrientation~=LANDSCAPE_RIGHT then
        fill(255,0,0)
        text("Rotate the ipad to Landscape mode.",WIDTH/2,HEIGHT/2)
        return
    end
        
    fill(255)
    text("Angular velocity (1)   "..w1.wheel.angularVelocity//-1,WIDTH/2,HEIGHT-25)
    text("Angular velocity (2)   "..w2.wheel.angularVelocity//-1,WIDTH/2,HEIGHT-50)
    text("Angular velocity (3)   "..w3.wheel.angularVelocity//-1,WIDTH/2,HEIGHT-75)
    text("Angular velocity (4)   "..w4.wheel.angularVelocity//-1,WIDTH/2,HEIGHT-100)
    text("Example of various physics joints",WIDTH/2,HEIGHT-150)
    
    w1:draw()
    w2:draw()
    w3:draw()
    w4:draw()
    drawPiston()
    drawJoints() 
    
    if r1.linearVelocity.x>0 then
        r1.linearVelocity=vec2(pressure*10,0)
    else
        r1.linearVelocity=vec2(pressure*-10,0)
    end
    
    if break_piston_joint and j5~=nil then   
        j5:destroy()
        j5=nil  -- no power to the wheels
        w1.wheel.angularDamping=1   -- slow all wheels
        w2.wheel.angularDamping=1
        w3.wheel.angularDamping=1
        w4.wheel.angularDamping=1
        pressure=0
    end
    if break_joint1_2 and j61~=nil then
        j61:destroy()
        j61=nil
        w1.wheel.angularDamping=1   -- slow wheel 1
    end
    if break_joint2_3 and j62~=nil then
        j62:destroy()
        j62=nil
        w2.wheel.angularDamping=1   -- slow wheel 2
        if j5==nil then -- piston joint broken
            w3.wheel.angularDamping=1   -- no power to wheel 3, so slow wheels 3 and 4
            w4.wheel.angularDamping=1
        end
    end
    if break_joint3_4 and j63~=nil then
        j63:destroy()
        j63=nil
        w3.wheel.angularDamping=1   -- slow wheels 3 and 4
        w4.wheel.angularDamping=1
    end   
end

function createPiston()
    p1=physics.body(CIRCLE,0)   -- anchor piston
    p1.x=100
    p1.y=350
    p1.type=STATIC
    
    r1=physics.body(CIRCLE,0)  -- piston rect
    r1.x=100
    r1.y=350
end

function drawPiston()
    if j5==nil then
        return
    end
    pushStyle()
    strokeWidth(2)
    rect(r1.x,r1.y,20,50) -- piston
    noFill()
    rect(120,350,160,50) -- piston cylinder
    rectMode(CORNER)
    if r1.linearVelocity.x>0 then
        fill(255,0,0,pressure+55)
        rect(p1.x-60,r1.y-25,r1.x-48,50)   -- forward direction, red pressure
    else
        fill(255,0,0,pressure+55)
        rect(r1.x+8,r1.y-25,p1.x-r1.x+92,50)   -- backward direction, red pressure
    end
    popStyle()
end

function drawJoints()
    pushStyle()
    strokeWidth(4)
    if j61~=nil then
        line(j61.bodyA.x,j61.bodyA.y,j61.bodyB.x,j61.bodyB.y)
    end
    if j62~=nil then
        line(j62.bodyA.x,j62.bodyA.y,j62.bodyB.x,j62.bodyB.y)
    end
    if j63~=nil then
        line(j63.bodyA.x,j63.bodyA.y,j63.bodyB.x,j63.bodyB.y)
    end
    if j5~=nil then
        line(j5.bodyA.x,j5.bodyA.y,j5.bodyB.x,j5.bodyB.y)
    end
    popStyle()
end

function createJoints()
    j31=physics.joint(REVOLUTE,w1.anchor1,w1.wheel,vec2(w1.anchor1.x,w1.anchor1.y))
    j32=physics.joint(REVOLUTE,w2.anchor1,w2.wheel,vec2(w2.anchor1.x,w2.anchor1.y))
    j33=physics.joint(REVOLUTE,w3.anchor1,w3.wheel,vec2(w3.anchor1.x,w3.anchor1.y))
    j34=physics.joint(REVOLUTE,w4.anchor1,w4.wheel,vec2(w4.anchor1.x,w4.anchor1.y))
    
    j41=physics.joint(REVOLUTE,w1.wheel,w1.anchor2,vec2(w1.wheel.x,w1.wheel.y+60))
    j42=physics.joint(REVOLUTE,w2.wheel,w2.anchor2,vec2(w2.wheel.x,w2.wheel.y+60))
    j43=physics.joint(REVOLUTE,w3.wheel,w3.anchor2,vec2(w3.wheel.x,w3.wheel.y+60))
    j44=physics.joint(REVOLUTE,w4.wheel,w4.anchor2,vec2(w4.wheel.x,w4.wheel.y+60))
        
    j61=physics.joint(DISTANCE,w1.anchor2,w2.anchor2,
        vec2(w1.anchor2.x,w1.anchor2.y),vec2(w2.anchor2.x,w2.anchor2.y))
    j62=physics.joint(DISTANCE,w2.anchor2,w3.anchor2,
        vec2(w2.anchor2.x,w2.anchor2.y),vec2(w3.anchor2.x,w3.anchor2.y))
    j63=physics.joint(DISTANCE,w3.anchor2,w4.anchor2,
        vec2(w3.anchor2.x,w3.anchor2.y),vec2(w4.anchor2.x,w4.anchor2.y))
    
    j1=physics.joint(PRISMATIC,p1,r1,p1.position,vec2(1,0))
    j5=physics.joint(DISTANCE,r1,w3.anchor2,r1.position,w3.anchor2.position)
end

wheel=class()

function wheel:init(a1x,a1y,wx,wy,a2x,a2y)
    self.anchor1=physics.body(CIRCLE,0)
    self.anchor1.x=a1x
    self.anchor1.y=a1y
    self.anchor1.type=STATIC
    
    self.wheel=physics.body(CIRCLE,80)
    self.wheel.x=wx
    self.wheel.y=wy
    
    self.anchor2=physics.body(CIRCLE,0)
    self.anchor2.x=a2x
    self.anchor2.y=a2y
end

function wheel:draw()
    pushMatrix()
    pushStyle()
    stroke(255)
    strokeWidth(2)
    noFill()
    translate(self.anchor1.x,self.anchor1.y)
    rotate(self.wheel.angle)
    ellipse(0,0,self.wheel.radius*2)       --large wheel
    ellipse(0,0,20)                 -- center circle
    ellipse(0,60,20)                -- outer circle
    for z=0,360,30 do
        x=math.cos(math.rad(z))*78
        y=math.sin(math.rad(z))*78
        line(0,0,x,y)
    end
    popStyle()
    popMatrix()
end

